;*******************************************************************************
; Themperatur Messung mit LCD Azeige fr 2 digitale Sensoren
; Ansteuerung zweier DS18B20 und Ausgabe der Temperatur auf dem LCD Display.
; Ausgabe Themperatur auch via RS232 9600BPS,n,8,1 
; RS232 Protokoll: Themperatur im klartext BCD, DS18x20 Register in HEX je 2x9Byte
;
; Programm:		LCD-Thermo
; Version:		V0.0
; Erstellt:		05.08.05
; System:		MCS51, CPU AT89C2051 @14.31818Hz (Timing DS18x20 10-16MHz ok)!
; Sprache:		Assembler
; Compiler:		www.batronix.com
; Author:		Manuel Magnin, Info@Clinch.ch, SYSTEM-CLINCH, CH-8355 Aadorf
;
; Anpassungen:	Version, Datum, Author, Beschreibung
;			V0.0, 05.08.05, MM, Grundversion
;
; Info:		CPU Quarz im Modul RS232 anpassen, damit die Baudrate stimmt!
;
; ToDo:		RS232 zur zeit nur Aausgabe, ev Menu mit Befehlen zur Abfrage??
;
; Routinen der Include Files
; RS232: RS232_INIT, RS232_SEND, RS232_GET, RS232_GET_BW, RS232_CRLF, RS232_SPACE, RS232_Hex, RS232_SEND_S
; LCD_4BIT: LCD_Init, LCD_Befehl, LCD_schreiben, LCD_prints
; DS1WIRE:  DS_Init, DS_senden, DS_lesen

INCLUDE 89C4051.MC 
 
; Konstanten-, Speicher- und Portbelegung
;---------------------------------------------------------------
StackP	    DATA 20h	'Start Adresse des Stack Pointers

LCD_RS          EQU P1.0	'RS Leitung des LCD
LCD_RW          EQU P1.1	'RW Leitung des LCD
LCD_ENABLE      EQU P1.2	'Enable Leitung des LCD
LCD_D4          EQU P1.4	'D4 Leitung des LCD
LCD_D5          EQU P1.5	'D5 Leitung des LCD
LCD_D6          EQU P1.6	'D6 Leitung des LCD
LCD_D7          EQU P1.7	'D7 Leitung des LCD

DQ		    EQU P3.4	'Dallas 1-Wire Signalleitung
DQ2		    EQU P3.5	'Dallas 1-Wire Signalleitung 2. Sensor

LED		    EQU P3.7	'LDE Anzeige Run/Idle

S20_Count	    DATA 08h	'DS18S20 Cout_Per_Celsius
S20_Remain	    DATA 0Ch	'DS18S20 Cout_Remain

TempV		    DATA 0Dh	'Vorzeichen der Temperatur	
Temp_E	    DATA 0Fh	'Wird fr die Berechnung bentigt
Temp_Z	    DATA 10h	' "
TempN		    DATA 11h	'Nachkommastelle der Temperatur
Temp	          DATA 12h	'Temperatur in Bin

TempL	          DATA 17h	'Vom Sensor gelesene, zu berechnende Hex Werte (Low)
TempH		    DATA 18h	'Vom Sensor gelesene, zu berechnende Hex Werte (High)

DS_ID           DATA 19h	'1-Wire Gerte ID
					'$10=DS18S20, $S8=DS18B20, $22=DS1822, $26=DS2438, $1D=DS2423
;Programmbeginn
;---------------------------------------------------------------  
 MOV SP,#StackP                 'StackPointer verschieben
 JMP start                      'Sprung zum eigentlichten Anfang des Programms
 INCLUDE wait.asm               'Einbindung der Wait Routinen
 INCLUDE lcd_4bit.asm           'Einbindung der LCD Routinen
 INCLUDE DS1WIRE.asm            'Einbindung der Dallas Routinen
 INCLUDE DS1WIRE2.asm           'Einbindung der Dallas Routinen 2. Sensor
 INCLUDE RS232.asm		  'Einbindung der RS232 Routinen

Temp_berechnen:
  MOV A,TempL     'Byte 0
  PUSH A		'Sichert A
  MOV B,#10h	'Division durch 16, Rest = Low-Nibble
  DIV AB		' "
  POP A		'Stellt A (Byte 0) wieder her
  MOV TempN,B	'Isoliert Low-Nibble
  CLR C		'Lscht gegbenfalls gesetztes C
  SUBB A,TempN	'Subbtrahiert Low-Nibble vom Byte um HighNibble zu Isolieren
  SWAP A		'Vertauscht High mit Low-Nibble
  MOV Temp_E,A	'Speichert Low-Nibble der Temperatur
  MOV A,TempH	'Byte 1
  PUSH A		'Sichert A
  MOV B,#10h	'Division durch 16, Rest = Low-Nibble 
  DIV AB		' "
  POP A		'Stellt A (Byte 1) wieder her
  CLR C		'Carry vor der Subtraktion Lschen	
  SUBB A,B		'High-Nibble Isolieren
  SWAP A		'Zu Low-Nibble Konvertieren
  MOV TempV,A	'und Speichern
  MOV A,B		'B in den Accu
  SWAP A		'und High mit Low-Nibble vertauschen
  MOV Temp_Z,A	'Wert Speichern
  ADD A,Temp_E	'Werte zusammenfhren
  MOV Temp,A	'und im Temp speichern
  IF TempV = #0Fh THEN
  MOV A,#FFh
  SUBB A,TEMP
  INC A
  MOV Temp,A
  END IF
RET

TempSend:
	MOV A,#":"		'Schreibt einen Doppelpunkt
      CALL RS232_Send
      CALL LCD_Schreiben
      
	IF TempV = #0Fh THEN	'Ist die Temperatur under Null dann wird das - geschieben.
	  MOV A,#"-"
	  CALL RS232_Send
        CALL LCD_Schreiben
	ELSE
	  MOV A,#" "		'sonnst eine leere Stelle
	  CALL RS232_Send
        CALL LCD_Schreiben
	END IF
	  
	MOV A,Temp 				;Zu Konvertierende Var. 

;Beginn BCD Decodierung
	CLR C 			;Lschen des Carry Bits 

	MOV R7,#2Fh 		;Beginne Hunderterstelle bei BCD "0" -1
_100:	SUBB A,#100 		;A = A-100 
	INC R7		 	;Hunderterstelle hochzhlen 
	JNC _100		 	;Wiederhole die beiden obigen Reihen, 
				  	'solange kein berlauf (C = 1) erfolgt ist. 
	PUSH A			;Akku retten
	MOV A,R7			;Resultat in Akku nehmen, Akku nach R7 retten
	IF A > #30h THEN 		'Ist die Zahl ber > "0" dann wird die 100ter Stelle geschrieben
        CALL LCD_Schreiben	'100ter Stelle geschrieben
        CALL RS232_Send		'100ter Stelle geschrieben
	END IF
	POP A				;Akku holen weiterrechnen
	
	ADD A,#100			;A =  A+100 (nach berlauf - Korrektur ) 
	CLR C 			;Lschen des Carry Bits 

	MOV R7,#2Fh 		;Beginne Zehnererstelle bei BCD "0" -1
_10:	SUBB A,#10 			;A = A-10 
	INC R7 			;Zehnerstelle hochzhlen 
	JNC _10		 	;Wiederhole die beiden obigen Reihen, 
					; solange kein berlauf (C = 1) erfolgt ist. 
	PUSH A			;Akku retten
	MOV A,R7			;Resultat in Akku nehmen, Akku nach R7 retten
	IF A > #30h THEN 		'Ist die Zahl ber > "0" dann wird die 10ter Stelle geschrieben
        CALL LCD_Schreiben	'100ter Stelle geschrieben
        CALL RS232_Send		'100ter Stelle geschrieben
	END IF
	POP A				;Akku holen weiterrechnen

_1_:	ADD A,#3Ah 			;Einerstelle (Korrektur berlauf von oben A = A+10 + BCD "0"
	CALL RS232_Send		;Ausgabe einer Stelle
	CALL LCD_Schreiben	;Ausgabe einer Stelle

	MOV A,#"." 			;Komma Trennen
	CALL RS232_Send		;Ausgabe einer Stelle
	CALL LCD_Schreiben	;Ausgabe einer Stelle

nachkomma:				'Berechnung der Nachkommastellen (Dividieren wie in der Grundschule)    
	MOV A,TempN			'A = TempN
	MOV B,#10h			'B =  10h
	DIV AB			'TempN durch 10h = 0, B = Rest
	MOV A,B			'A = Rest (B)
	MOV B,#0Ah			'B = 10d = 0Ah
	MUL AB			'A mal 10 
nachkomma2:
	MOV B,#10h			'hier wird das jetzt so weiter gefhrt,
	DIV AB			'bis "unterm Streich" 0 rauskommt
	ADD A,#30h
	CALL LCD_schreiben	'jede berechnete Nachkommastelle wird direkt
	CALL RS232_Send		'jede berechnete Nachkommastelle wird direkt
	MOV A,B			'auf das Display geschreiben.
	MOV B,#0Ah
	MUL AB
	MOV TempN,A
	IF TempN <> #00h THEN JMP nachkomma2
		  
	MOV A,#DFh			'Schreibt das  Zeichen auf dem LCD
	CALL LCD_schreiben
	MOV A,#"C"			'Schreibt das C Zeichen auf dem LCD
	CALL RS232_Send		;Ausgabe RS232
	CALL LCD_schreiben
	MOV A,#" "			'Schreibt ein Space Zeichen auf dem LCD alte Zeichen lschen
	CALL LCD_schreiben
	CALL RS232_Send		;Ausgabe RS232
	MOV A,#" "			'Schreibt ein Space Zeichen auf dem LCD alte Zeichen lschen
	CALL LCD_schreiben
	MOV A,#" "			'Schreibt ein Space Zeichen auf dem LCD alte Zeichen lschen
	CALL LCD_schreiben
      RET


DS18S20_HiRes:			'Hohe Genauigkeit nachrechnen fr DS18S20
        IF DS_ID = #10h THEN	'Ist der Sensor ein DS18S20 nicht B20 dann sind nur 9 Bit da anst 12!
	    MOV	A,TempL	'Ganze Temperatur um 3 Bit Schieben ( mal 8 rechnen )
	    SWAP A
	    RR A
	    ANL A,#07h
	    MOV TempH,A		'High Byte ok
	    MOV A,TempL		'Low Byte 3 Bit hher Werten
	    RL A
	    RL A
	    RL A
	    ANL A,#0F0h		'Die Nachkomma Stellen abschneiden, Nicht nach Handbuch
	    MOV TempL,A		'Komma Stellen jetzt in den letzten 4 Bits wie DS18B20
					'Nachkomma Stellen 4 Bits jetzt nachrechnen!!
          MOV A,S20_Count	'DS18S20 Cout_Per_Celsius 
          SUBB A,S20_Remain	'DS18S20 Cout_Remain
          SWAP A			'Multiplikation mit 16 (nicht 0 .. 0.99 sonder 0 ..15 als Wertebereich)
	    MOV B,S20_Count	'Total Zhler pro Grad Celsius
          DIV AB			'Division durch S20_Count, Rest = B, Akku = Ganzzahl
	    ADD A,TempL		'Nachkommastellen zu TempL anfgen
	    MOV TempL,A
	  END IF
	RET


Wait1S:   MOV R5,#50
wait1S1:  MOV R6,#215		'R6 = 215
wait1s2:  MOV R7,#45		'R7 = 45
	    DJNZ R7,$		'R7--: If (R7 > 0) repeat
	    DJNZ R6,wait1s2	'R6--: If (R6 > 0) _wait_s10
	    DJNZ R5,wait1s1	'R5--: If (R5 > 0) _wait_s9
	    MOV R7,#47		'R7 = 47
	    DJNZ R7,$		'R7--: If (R7 > 0) repeat
	    RET			'Rcksprung




start:  CALL LCD_Init           'LC-Display Ansteuerung
        
;	  MOV DPTR,#Zeile1	  'Aufruf des Strings Zeile1	  
;	  CALL LCD_prints		  'Schreiben eines Strings

	  CALL RS232_INIT		  'Initiert die RS232 Schnittstelle
	  
	  MOV DPTR,#StartText	  'Aufruf des Strings Zeile1	  
	  CALL RS232_SEND_S	  'Schreiben eines Strings


Again:  MOV A,#LCD_Zeile1 	  'Sprung zur ersten Stelle der zweiten Zeile
	  CALL LCD_Befehl
	  
	  CALL DS_Init                       '1-Wire Bus berprfung auf Komponenten
	  MOV A,#"1"				 'Ist ein Bauteil im Bus vorhanden Gibt das display
	  IF BIT C THEN MOV A,#"F"     	 'eine "1" aus, ist keins Vorhanden dann eine "F"
	  CALL LCD_schreiben
        MOV A,#"-"	            'Schreibt einen Bindestrich
        CALL LCD_Schreiben

        MOV A,#33h		'1-Wire ROM Befehl
	  CALL DS_senden		'1-Wire 1 Byte-Senden
	  CALL DS_lesen		'1. Byte lesen - Famili Code
	  MOV DS_ID,A		'Speichert das Type Byte
        CALL LCD_HEX          'ID Hex ausgeben auf LCD

 	  CALL DS_Init          '1-Wire Bus berprfung auf Komponenten
	  MOV A,#CCh		'Rom Command: Skip ROM
	  CALL DS_senden
	  MOV A,#44h		'Temp-Convert Befehl, des DS18B20
	  CALL DS_senden

	  CALL Wait1S		'Warten 1S Sek. - Parasit Power Condensator im DS18x20 Laden

	  CALL DS_init		'Start
	  MOV A,#CCh		'Rom Command: Skip Rom
	  CALL DS_senden
	  MOV A,#BEh		'Functions Command: Read PAD
	  Call DS_senden

	  CALL DS_lesen		'1. Byte 0 Lesen
	  MOV TempL,A		'Speichert das LOW Byte der Temperatur
	  CALL DS_Lesen		'2. Byte 1 Lesen
	  MOV TempH,A		'Speichert das High Byte der Temperatur
	  CALL DS_Lesen		'3. Byte 1 Lesen
	  CALL DS_Lesen		'4. Byte 1 Lesen
	  CALL DS_Lesen		'5. Byte 1 Lesen
	  CALL DS_Lesen		'6. Byte 1 Lesen
	  CALL DS_Lesen		'7. Byte 1 Lesen
	  MOV S20_Remain,A	'DS18S20 Zhler rest sichern
	  CALL DS_Lesen		'8. Byte 1 Lesen
        MOV S20_Count,A		'DS18S20 Zhler total Cout_Per_Celsius

	  Call DS18S20_HiRes    'Hohe Genauigkeit nachrechnen
        Call Temp_berechnen	'Berechnet die Temperatur aus High und Low Byte  
        CALL TempSend		'Andre's umwandlung von Hex in Dezimalzahlen.

;** Zweite Zeile *********************************************************************************
	  MOV A,#LCD_Zeile2 	  'Sprung zur ersten Stelle der zweiten Zeile
	  CALL LCD_Befehl
	  
	  CALL DS_Init2               '1-Wire Bus berprfung auf Komponenten
	  MOV A,#"2"			'Ist ein Bauteil im Bus vorhanden Gibt das display
	  IF BIT C THEN MOV A,#"F"    'eine "2" aus, ist keins Vorhanden dann eine "F"
	  CALL LCD_schreiben
        MOV A,#"-"	            'Schreibt einen Bindestrich
        CALL LCD_Schreiben

        MOV A,#33h			'1-Wire ROM Befehl
	  CALL DS_senden2			'1-Wire 1 Byte-Senden
	  CALL DS_lesen2		'1. Byte lesen - Famili Code
	  MOV DS_ID,A		'Speichert das Type Byte
        CALL LCD_HEX          'ID Hex ausgeben auf LCD

 	  CALL DS_Init2         '1-Wire Bus berprfung auf Komponenten
	  MOV A,#CCh		'Rom Command: Skip ROM
	  CALL DS_senden2
	  MOV A,#44h		'Temp-Convert Befehl, des DS18B20
	  CALL DS_senden2

	  CALL Wait1S		'Warten 1S Sek. - Parasit Power Condensator im DS18x20 Laden

	  CALL DS_init2		'Start
	  MOV A,#CCh		'Rom Command: Skip Rom
	  CALL DS_senden2
	  MOV A,#BEh		'Functions Command: Read PAD
	  Call DS_senden2
	  
	  CALL DS_lesen2		'1. Byte 0 Lesen
	  MOV TempL,A		'Speichert das LOW Byte der Temperatur
	  CALL DS_Lesen2		'2. Byte 1 Lesen
	  MOV TempH,A		'Speichert das High Byte der Temperatur
	  CALL DS_Lesen2		'3. Byte 1 Lesen
	  CALL DS_Lesen2		'4. Byte 1 Lesen
	  CALL DS_Lesen2		'5. Byte 1 Lesen
	  CALL DS_Lesen2		'6. Byte 1 Lesen
	  CALL DS_Lesen2		'7. Byte 1 Lesen
	  MOV S20_Remain,A	'DS18S20 Zhler rest sichern
	  CALL DS_Lesen2		'8. Byte 1 Lesen
        MOV S20_Count,A		'DS18S20 Zhler total Cout_Per_Celsius

	  Call DS18S20_HiRes    'Hohe Genauigkeit nachrechnen
        Call Temp_berechnen	'Berechnet die Temperatur aus High und Low Byte  
        CALL TempSend		'Andre's umwandlung von Hex in Dezimalzahlen.


;** RS232 Ausgabe ******************************************************************************      				

	  CALL DS_Init          '1-Wire Bus berprfung auf Komponenten   
	  CALL DS_Init2         '1-Wire Bus berprfung auf Komponenten   
	  MOV A,#33h		'1-Wire ROM Befehl
	  CALL DS_senden		'1-Wire 1 Byte-Senden
	  CALL DS_senden2		'1-Wire 1 Byte-Senden
        
        MOV R2,#09
DSROM1: CALL DS_lesen		'alle 9 Bytes lesen
	  CALL RS232_HEX        'Ausgabe Seriel Byte als HEX          
	  DJNZ R2,DSROM1
        call RS232_SPACE	'Trennen der Blcke

        MOV R2,#09
DSROM2: CALL DS_lesen2		'alle 9 Bytes lesen
	  CALL RS232_HEX        'Ausgabe Seriel Byte als HEX          
	  DJNZ R2,DSROM2
        call RS232_SPACE	'Trennen der Blcke



 	  CALL DS_Init          '1-Wire Bus berprfung auf Komponenten
 	  CALL DS_Init2         '1-Wire Bus berprfung auf Komponenten
	  MOV A,#CCh		'Rom Command: Skip ROM
	  CALL DS_senden
	  CALL DS_senden2
	  MOV A,#44h		'Temp-Convert Befehl, des DS18B20
	  CALL DS_senden
	  CALL DS_senden2

	  CALL Wait1S		'Warten 1S Sek. - Parasit Power Condensator im DS18x20 Laden
	  
	  CALL DS_init		'Start
	  CALL DS_init2		'Start
	  MOV A,#CCh		'Rom Command: Skip Rom
	  CALL DS_senden
	  CALL DS_senden2
	  MOV A,#BEh		'Functions Command: Read PAD
	  Call DS_senden
	  Call DS_senden2
	  
        MOV R2,#09
DSTEM1: CALL DS_lesen		'alle 9 Bytes lesen
	  CALL RS232_HEX        'Ausgabe Seriel Byte als HEX 
        DJNZ R2,DSTEM1
        call RS232_SPACE	'Trennen der Blcke

        MOV R2,#09
DSTEM2: CALL DS_lesen2		'alle 9 Bytes lesen
	  CALL RS232_HEX        'Ausgabe Seriel Byte als HEX 
        DJNZ R2,DSTEM2
	  Call RS232_CRLF

      	
	  SETB LED			'LDE ausschalten Idle
	  CALL Wait1S		'Warte 2 Sekunden Wartezeit bis eine neue Temperaturmessung erfolgt.
	  CALL Wait1S		'   "
	  CLR LED			'LED einschalten Run
	  JMP Again			'Nchste Messung

StartText: DB "DS18x20 mit LCD, RS232",0





